@@ -17,10 +17,10 @@ BEGIN
 
 use SDL::App::FPS::Demo;
 
-# fill in here options or use Getopt::Long for command line
 my $options = { width => 640, height => 480, max_fps => 40};
 
-print "SDL::App::FPS Demo v0.01 (C) 2002 by Tels <http://Bloodgate.com/>\n\n";
+print
+  "SDL::App::FPS Demo v0.01 (C) 2002,2003 by Tels <http://Bloodgate.com/>\n\n";
 
 my $app = SDL::App::FPS::Demo->new( $options );
 $app->main_loop();
